@@ -163,6 +163,8 @@ struct GameInfo
   boolean use_masked_pushing;
   boolean use_masked_elements;
   boolean use_masked_elements_initial;
+  int forced_scroll_x;
+  int forced_scroll_y;
   int forced_scroll_delay_value;
   int scroll_delay_value;
   int tile_size;
@@ -203,7 +205,8 @@ struct GameInfo
 
   boolean explosions_delayed;
   boolean envelope_active;
-  boolean no_time_limit;	// (variable only in very special case)
+  boolean no_level_time_limit;	// (variable only in very special case)
+  boolean time_limit;		// forced by levelset config or setup option
 
   int time_final;		// time (in seconds) or steps left or played
   int score_time_final;		// time (in frames) or steps played
@@ -241,12 +244,8 @@ struct GameInfo
   // values for special game initialization control
   boolean restart_level;
 
-  // trigger message to ask for restarting the game
-  char *restart_game_message;
-
   // values for special request dialog control
   boolean request_active;
-  boolean request_active_or_moving;
 
   // values for special game control
   int centered_player_nr;
@@ -276,6 +275,8 @@ struct GameInfo
   int LevelSolved_CountingTime;
   int LevelSolved_CountingScore;
   int LevelSolved_CountingHealth;
+
+  boolean RestartGameRequested;
 };
 
 struct PlayerInfo
@@ -378,7 +379,7 @@ struct PlayerInfo
   int push_delay;
   int push_delay_value;
 
-  unsigned int actual_frame_counter;
+  DelayCounter actual_frame_counter;
 
   int drop_delay;
   int drop_pressed_delay;
@@ -426,8 +427,10 @@ void GameEnd(void);
 void MergeServerScore(void);
 
 void InitPlayerGfxAnimation(struct PlayerInfo *, int, int);
+
 void Moving2Blocked(int, int, int *, int *);
 void Blocked2Moving(int, int, int *, int *);
+
 void DrawDynamite(int, int);
 
 void StartGameActions(boolean, boolean, int);
@@ -454,9 +457,8 @@ void RaiseScoreElement(int);
 
 void RequestQuitGameExt(boolean, boolean, char *);
 void RequestQuitGame(boolean);
-void RequestRestartGame(char *);
-void CheckGameOver(void);
 
+boolean CheckRestartGame(void);
 boolean checkGameSolved(void);
 boolean checkGameFailed(void);
 boolean checkGameEnded(void);
