@@ -64,39 +64,39 @@
 #define VIDEO_CONTROL_YSIZE	VIDEO_BUTTON_YSIZE
 
 // values for video tape control
-#define VIDEO_STATE_PLAY_OFF	(1 << 0)
-#define VIDEO_STATE_PLAY_ON	(1 << 1)
-#define VIDEO_STATE_REC_OFF	(1 << 2)
-#define VIDEO_STATE_REC_ON	(1 << 3)
-#define VIDEO_STATE_PAUSE_OFF	(1 << 4)
-#define VIDEO_STATE_PAUSE_ON	(1 << 5)
-#define VIDEO_STATE_DATE_OFF	(1 << 6)
-#define VIDEO_STATE_DATE_ON	(1 << 7)
-#define VIDEO_STATE_TIME_OFF	(1 << 8)
-#define VIDEO_STATE_TIME_ON	(1 << 9)
-#define VIDEO_STATE_FRAME_OFF	(1 << 10)
-#define VIDEO_STATE_FRAME_ON	(1 << 11)
-#define VIDEO_STATE_FFWD_OFF	(1 << 12)
-#define VIDEO_STATE_FFWD_ON	(1 << 13)
-#define VIDEO_STATE_WARP_OFF	(1 << 14)
-#define VIDEO_STATE_WARP_ON	(1 << 15)
-#define VIDEO_STATE_WARP2_OFF	(1 << 16)
-#define VIDEO_STATE_WARP2_ON	(1 << 17)
-#define VIDEO_STATE_PBEND_OFF	(1 << 18)
-#define VIDEO_STATE_PBEND_ON	(1 << 19)
-#define VIDEO_STATE_1STEP_OFF	(1 << 20)
-#define VIDEO_STATE_1STEP_ON	(1 << 21)
+#define VIDEO_STATE_PLAY_OFF	(1u << 0)
+#define VIDEO_STATE_PLAY_ON	(1u << 1)
+#define VIDEO_STATE_REC_OFF	(1u << 2)
+#define VIDEO_STATE_REC_ON	(1u << 3)
+#define VIDEO_STATE_PAUSE_OFF	(1u << 4)
+#define VIDEO_STATE_PAUSE_ON	(1u << 5)
+#define VIDEO_STATE_DATE_OFF	(1u << 6)
+#define VIDEO_STATE_DATE_ON	(1u << 7)
+#define VIDEO_STATE_TIME_OFF	(1u << 8)
+#define VIDEO_STATE_TIME_ON	(1u << 9)
+#define VIDEO_STATE_FRAME_OFF	(1u << 10)
+#define VIDEO_STATE_FRAME_ON	(1u << 11)
+#define VIDEO_STATE_FFWD_OFF	(1u << 12)
+#define VIDEO_STATE_FFWD_ON	(1u << 13)
+#define VIDEO_STATE_WARP_OFF	(1u << 14)
+#define VIDEO_STATE_WARP_ON	(1u << 15)
+#define VIDEO_STATE_WARP2_OFF	(1u << 16)
+#define VIDEO_STATE_WARP2_ON	(1u << 17)
+#define VIDEO_STATE_PBEND_OFF	(1u << 18)
+#define VIDEO_STATE_PBEND_ON	(1u << 19)
+#define VIDEO_STATE_1STEP_OFF	(1u << 20)
+#define VIDEO_STATE_1STEP_ON	(1u << 21)
 
-#define VIDEO_PRESS_PLAY_ON	(1 << 22)
-#define VIDEO_PRESS_PLAY_OFF	(1 << 23)
-#define VIDEO_PRESS_REC_ON	(1 << 24)
-#define VIDEO_PRESS_REC_OFF	(1 << 25)
-#define VIDEO_PRESS_PAUSE_ON	(1 << 26)
-#define VIDEO_PRESS_PAUSE_OFF	(1 << 27)
-#define VIDEO_PRESS_STOP_ON	(1 << 28)
-#define VIDEO_PRESS_STOP_OFF	(1 << 29)
-#define VIDEO_PRESS_EJECT_ON	(1 << 30)
-#define VIDEO_PRESS_EJECT_OFF	(1 << 31)
+#define VIDEO_PRESS_PLAY_ON	(1u << 22)
+#define VIDEO_PRESS_PLAY_OFF	(1u << 23)
+#define VIDEO_PRESS_REC_ON	(1u << 24)
+#define VIDEO_PRESS_REC_OFF	(1u << 25)
+#define VIDEO_PRESS_PAUSE_ON	(1u << 26)
+#define VIDEO_PRESS_PAUSE_OFF	(1u << 27)
+#define VIDEO_PRESS_STOP_ON	(1u << 28)
+#define VIDEO_PRESS_STOP_OFF	(1u << 29)
+#define VIDEO_PRESS_EJECT_ON	(1u << 30)
+#define VIDEO_PRESS_EJECT_OFF	(1u << 31)
 
 #define VIDEO_STATE_PLAY(x)  ((x) ? VIDEO_STATE_PLAY_ON : VIDEO_STATE_PLAY_OFF)
 #define VIDEO_STATE_REC(x)   ((x) ? VIDEO_STATE_REC_ON  : VIDEO_STATE_REC_OFF)
@@ -210,6 +210,7 @@ struct TapeInfo
   boolean quick_resume;
   boolean single_step;
   boolean changed;
+  boolean solved;
   boolean player_participates[MAX_PLAYERS];
   int num_participating_players;
   int centered_player_nr_next;
@@ -256,8 +257,8 @@ void TapeSetDateFromNow(void);
 void TapeStartRecording(int);
 void TapeHaltRecording(void);
 void TapeStopRecording(void);
-boolean TapeAddAction(byte *);
-void TapeRecordAction(byte *);
+boolean TapeAddAction(byte[MAX_TAPE_ACTIONS]);
+void TapeRecordAction(byte[MAX_TAPE_ACTIONS]);
 void TapeTogglePause(boolean);
 void TapeStartPlaying(void);
 void TapeStopPlaying(void);
