@@ -63,8 +63,6 @@
 #define REQ_STAY_CLOSED		(1 << 4)
 #define REQ_REOPEN		(1 << 5)
 
-#define REQUEST_WAIT_FOR_INPUT	(REQ_ASK | REQ_CONFIRM | REQ_PLAYER)
-
 
 int getFieldbufferOffsetX_RND(int, int);
 int getFieldbufferOffsetY_RND(int, int);
@@ -85,7 +83,7 @@ void DrawMaskedBorder_DOOR_3(void);
 void DrawMaskedBorder_ALL(void);
 void DrawMaskedBorder(int);
 void DrawMaskedBorderToTarget(int);
-void DrawTileCursor(int);
+void DrawTileCursor(int, int);
 
 void SetDrawtoField(int);
 int GetDrawtoField(void);
@@ -133,11 +131,13 @@ void FloodFillLevel(int, int, int, short[MAX_LEV_FIELDX][MAX_LEV_FIELDY], int, i
 void FloodFillLevelExt(int, int, int, int x, int y, short field[x][y], int, int);
 
 void SetRandomAnimationValue(int, int);
+void SetAnimationFirstLevel(int);
 int getGraphicAnimationFrame(int, int);
 int getGraphicAnimationFrameXY(int, int, int);
 
 void DrawFixedGraphicAnimation(int, int, int);
 void DrawFixedGraphicAnimationExt(DrawBuffer *, int, int, int, int, int);
+void DrawSizedGraphicAnimationExt(DrawBuffer *, int, int, int, int, int, int);
 
 void DrawLevelGraphicAnimation(int, int, int);
 void DrawLevelElementAnimation(int, int, int);
@@ -202,7 +202,7 @@ void DrawMiniElementOrWall(int, int, int, int);
 
 void ShowEnvelope(int);
 void ShowEnvelopeDoor(char *, int);
-void DrawEnvelopeRequestToScreen(int, int);
+void DrawEnvelopeRequestToScreen(int);
 
 void DrawLevel(int);
 void DrawSizedLevel(int, int, int, int, int);
@@ -215,7 +215,7 @@ void DrawNetworkPlayers(void);
 void ClearNetworkPlayers(void);
 
 void WaitForEventToContinue(void);
-boolean Request(char *, unsigned int);
+int Request(char *, unsigned int);
 void InitGraphicCompatibilityInfo_Doors(void);
 void InitDoors(void);
 unsigned int OpenDoor(unsigned int);
